/*
 Stephan Kottler
 kottlers@informatik.uni-tuebingen.de
 *
 */

#ifndef STATISTICS_HPP_
#define STATISTICS_HPP_

#include <sys/resource.h>
#include <stdint.h>
#include <unistd.h>
#include <vector>
#include <string>

#include "Settings.hpp"
#include "OFStream.hpp"

class ConstraintDB;

/** Class to store all kind of information during the solving process.*/
struct Statistics{

	Statistics(Settings& sets);

	Settings&				settings;

	uint64_t				nbr_conflicts;

	uint64_t				nbr_propagations;

	uint64_t				nbr_act_decisions;

	uint64_t				nbr_rnd_decisions;

	uint32_t				nbr_restarts;

	//uint32_t				nbr_local_jumpbacks;

	uint32_t				max_backjumping_distance;

	uint64_t				sum_backjumping;

	uint32_t				nbr_expensive_cls_bmp;

	uint32_t				nbr_expensive_var_bmp;

	uint32_t 				nbr_flipped_prop_learnts;

	uint32_t				nbr_learnt_reductions;

	uint64_t				total_lemma_shortening;

	uint32_t				nbr_simplify_execs;

	uint32_t				nbr_removed_cls_lits;

	uint32_t				nbr_simplify_worked;

	uint32_t 				nbr_bigsimplify_shrinked;

	uint32_t				max_dec_level;

	uint32_t				nbr_expensive_cls_stamp;

	uint32_t				nbr_panic_gc;

	double					parse_timestmp;

	double					finish_timestmp;

	/**Logs the backjumping distance for the detailed statistics.
	* @param dist: the number of levels that were skipped due to backjumping*/
	void					storeBackjumpingDistance(const int dist);

	/**Print all statistic information to the given OFStream.
	* @param out: the stream to printDescription to*/
	void					printFinalStatistics(OFStream& out, const ConstraintDB* sp = NULL);

	/**Print all statistic information to settings.console_out
	* @param out: the stream to printDescription to*/
	void					printStatsProgress(uint64_t literals);


	/**Print all statistic information to the given OFStream.
	* @param out: the stream to printDescription to*/
	void					printStatsProgressHead();



	/**Print all statistic information into a comma seperated file.
	* Instead of a seperation by commata the character '&' is used to avoid clashes with decimal commata for local printing.
	* @param out: the stream to printDescription to
	* @param result: the result of the solving process (SAT/UNSAT/Trivial UNSAT ...)*/
	void					printCSV(OFStream& out,const ConstraintDB* sp, char* result);

	void					printGnuPlot(OFStream& out,const ConstraintDB* sp, char* result);

	/**Get the system time that was used by the cpu for this process.
	* @return the system time in seconds*/
	double					cpuSystemTime();

	/**Get the user time that was used by the cpu for this process.
	* @return the user time in seconds.*/
	double					cpuUserTime();

	/**Get the amount of bytes that were used by this process.
	* @return the amount of bytes (in bytes)*/
	uint64_t				usedMemoryBytes();

	/**CVS Version numbers as string*/
	void					printCVSVersionNumbers(OFStream& out, const char* first = "", const char* sep = "");

protected:
	void					printFile(OFStream& out,const char* seperator, const ConstraintDB* sp, char* result);

};



#define statistics_hpp_version "Statistics.hpp:	$Revision: 1.6 $"
#endif /*STATISTICS_HPP_*/
